/*
 * BrowserView.java - A VFS browser file view
 * Copyright (C) 2000 Slava Pestov
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.gjt.sp.jedit.browser;

import javax.swing.JPanel;
import java.awt.*;
import java.util.Vector;
import org.gjt.sp.jedit.io.VFS;

/**
 * A browser view.
 * @author Slava Pestov
 * @version $Id: BrowserView.java,v 1.5 2000/08/29 07:47:12 sp Exp $
 */
public abstract class BrowserView extends JPanel
{
	public BrowserView(VFSBrowser browser)
	{
		this.browser = browser;
	}

	/**
	 * Returns the currently selected files.
	 */
	public abstract VFS.DirectoryEntry[] getSelectedFiles();

	/**
	 * A directory has been loaded.
	 * @param directory The directory listing
	 */
	public abstract void directoryLoaded(Vector directory);

	/**
	 * Called when a buffer is opened or closed. Views that reflect
	 * this should update themselves accordingly in this method.
	 */
	public abstract void updateFileView();

	/**
	 * So that the tree view can update only the appropriate branch,
	 * instead of the whole tree.
	 */
	public abstract void reloadDirectory(String path);

	/**
	 * Returns the component that is to have focus by default.
	 */
	public abstract Component getDefaultFocusComponent();

	// protected members
	protected VFSBrowser browser;

	protected void showFilePopup(VFS.DirectoryEntry file, Component comp, Point p)
	{
		BrowserPopupMenu popup = new BrowserPopupMenu(browser,file);
		popup.show(comp,p.x,p.y);
	}
}

/*
 * Change Log:
 * $Log: BrowserView.java,v $
 * Revision 1.5  2000/08/29 07:47:12  sp
 * Improved complete word, type-select in VFS browser, bug fixes
 *
 * Revision 1.4  2000/08/06 09:44:27  sp
 * VFS browser now has a tree view, rename command
 *
 * Revision 1.3  2000/08/05 07:16:12  sp
 * Global options dialog box updated, VFS browser now supports right-click menus
 *
 * Revision 1.2  2000/07/31 11:32:09  sp
 * VFS file chooser is now in a minimally usable state
 *
 * Revision 1.1  2000/07/30 09:04:19  sp
 * More VFS browser hacking
 *
 */
