/*
 * SearchFileSet.java - Abstract file matcher interface
 * Copyright (C) 1999 Slava Pestov
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.gjt.sp.jedit.search;

import org.gjt.sp.jedit.*;

/**
 * An abstract interface for matching files.
 * @author Slava Pestov
 * @version $Id: SearchFileSet.java,v 1.8 2000/05/14 10:55:22 sp Exp $
 */
public interface SearchFileSet
{
	/**
	 * Returns the first buffer to search.
	 * @param view The view performing the search
	 */
	Buffer getFirstBuffer(View view);

	/**
	 * Returns the next buffer to search.
	 * @param view The view performing the search
	 * @param buffer The last buffer searched
	 */
	Buffer getNextBuffer(View view, Buffer buffer);

	/**
	 * Called if the specified buffer was found to have a match.
	 * @param buffer The buffer
	 */
	void matchFound(Buffer buffer);

	/**
	 * Returns the number of buffers in this file set.
	 */
	int getBufferCount();
}
/*
 * ChangeLog:
 * $Log: SearchFileSet.java,v $
 * Revision 1.8  2000/05/14 10:55:22  sp
 * Tool bar editor started, improved view registers dialog box
 *
 * Revision 1.7  1999/11/28 00:33:07  sp
 * Faster directory search, actions slimmed down, faster exit/close-all
 *
 * Revision 1.6  1999/11/21 01:20:31  sp
 * Bug fixes, EditBus updates, fixed some warnings generated by jikes +P
 *
 * Revision 1.5  1999/10/10 06:38:45  sp
 * Bug fixes and quicksort routine
 *
 * Revision 1.4  1999/10/02 01:12:36  sp
 * Search and replace updates (doesn't work yet), some actions moved to TextTools
 *
 * Revision 1.3  1999/06/09 07:28:10  sp
 * Multifile search and replace tweaks, removed console.html
 *
 * Revision 1.2  1999/06/09 05:22:11  sp
 * Find next now supports multi-file searching, minor Perl mode tweak
 *
 * Revision 1.1  1999/06/03 08:43:57  sp
 * Adding missing file
 *
 */
